<?php
// Own namespace
namespace Org\Mxchange\City\Controller\LoginArea;

// Import framework stuff
use Org\Mxchange\CoreFramework\Controller\BaseController;
use Org\Mxchange\CoreFramework\Controller\Controller;
use Org\Mxchange\CoreFramework\Factory\ObjectFactory;
use Org\Mxchange\CoreFramework\Request\Requestable;
use Org\Mxchange\CoreFramework\Resolver\Command\CommandResolver;
use Org\Mxchange\CoreFramework\Response\Responseable;

/**
 * Controller for login area with news
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2015, 2016 City Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class CityHtmlLoginAreaController extends BaseController implements Controller {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Init additional filter chains
		foreach (array('shutdown') as $filterChain) {
			$this->initFilterChain($filterChain);
		} // END - foreach
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$resolverInstance		An instance of a command resolver class
	 * @return	$controllerInstance		A prepared instance of this class
	 * @todo	Add some morer filters to this controller
	 */
	public static final function createCityHtmlLoginAreaController (CommandResolver $resolverInstance) {
		// Create the instance
		$controllerInstance = new CityHtmlLoginAreaController();

		// Set the command resolver
		$controllerInstance->setResolverInstance($resolverInstance);

		// User auth filter
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('user_auth_filter'));

		// User update filter
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('user_update_filter'));

		// News fetcher filter
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('news_download_filter'));

		// News proccess/display-preparation
		$controllerInstance->addPreFilter(ObjectFactory::createObjectByConfiguredName('news_process_filter'));

		// Return the prepared instance
		return $controllerInstance;
	}

	/**
	 * Handles the given request and response
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	public function handleRequest (Requestable $requestInstance, Responseable $responseInstance) {
		/*
		 * Generic execute of the command: pre and post filters with redirect
		 * but request becomes valid after pre-filters run.
		 */
		$this->genericHanleRequestLoginAreaFailedRedirect($requestInstance, $responseInstance);
	}
}
