<?php
// Own namespace
namespace Org\Mxchange\City\Factory\Manager;

// Import framework stuff
use Org\Mxchange\CoreFramework\Factory\ObjectFactory;
use Org\Mxchange\CoreFramework\Registry\Registry;

/**
 * A factory class for managers
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2015, 2016 City Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ManagerFactory extends ObjectFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a singleton (registry-based) ManageableFoo instance
	 *
	 * @param	$type				Type of manager to return
	 * @return	$managerInstance	An instance of a ManageableFoo class
	 */
	public static final function createManagerByType ($type) {
		// Get new factory instance
		$factoryInstance = new ManagerFactory();

		// Generate key
		$key = $type . '_manager';

		// If there is no handler?
		if (Registry::getRegistry()->instanceExists($key)) {
			// Get handler from registry
			$managerInstance = Registry::getRegistry()->getInstance($key);
		} else {
			// Get the proper manager instance
			$managerInstance = ObjectFactory::createObjectByConfiguredName($key . '_class');

			// Add it to the registry
			Registry::getRegistry()->addInstance($key, $managerInstance);
		}

		// Return the instance
		return $managerInstance;
	}
}
