<?php
// Own namespace
namespace Org\Mxchange\City\Daemon\Task\Growth\Building;

// Import application-specific stuff
use Org\Mxchange\City\Daemon\Factory\CityDaemonFactory;
use Org\Mxchange\City\Task\BaseCityTask;

// Import framework stuff
use Org\Mxchange\CoreFramework\Task\Taskable;
use Org\Mxchange\CoreFramework\Visitor\Visitable;
use Org\Mxchange\CoreFramework\Visitor\Visitor;

/**
 * A BuildingGrowth city daemon-task
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2015, 2016 City Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class CityDaemonBuildingGrowthTask extends BaseCityTask implements Taskable, Visitable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$taskInstance	An instance of a Visitable class
	 */
	public final static function createCityDaemonBuildingGrowthTask () {
		// Get new instance
		$taskInstance = new CityDaemonBuildingGrowthTask();

		// Get city daemon instance
		$cityInstance = CityDaemonFactory::createCityDaemonInstance();

		// Set it here for "caching" it
		$taskInstance->setCityInstance($cityInstance);

		// Return the prepared instance
		return $taskInstance;
	}

	/**
	 * Accepts the visitor to process the visitor
	 *
	 * @param	$visitorInstance	An instance of a Visitor class
	 * @return	void
	 * @todo	Maybe visit some sub-objects
	 */
	public function accept (Visitor $visitorInstance) {
		// Visit this task
		$visitorInstance->visitTask($this);
	}

	/**
	 * Executes the task
	 *
	 * @return	void
	 * @todo	0% done
	 */
	public function executeTask () {
		// Get daemon instance
		$cityInstance = $this->getCityInstance();

		// Is there at least one building to expand?
		if ($cityInstance->isBuildingPendingExpansion()) {
			// Expand the building
			$cityInstance->expandBuildings();
		} // END - if
	}

	/**
	 * Shuts down the task
	 *
	 * @return	void
	 * @todo	0% done
	 */
	public function doShutdown () {
		self::createDebugInstance(__CLASS__)->debugOutput('TASK[' . __METHOD__ . ':' . __LINE__ . ']: Shutting down...');
	}
}
